searchNodes=[{"doc":"A helper for running functions that might take too long, or use too much memory. Handler will run these functions in their own process and &quot;take care of&quot; problematic processes.","ref":"Handler.html","title":"Handler","type":"module"},{"doc":"Run a potentially dangerous function in a safe way. Examples iex&gt; Handler . run ( fn -&gt; 1 + 1 end ) 2 iex&gt; Handler . run ( fn -&gt; :timer . sleep ( 200 ) end , max_ms : 10 ) { :error , % Handler.Timeout { message : &quot;Took more than 10ms to complete&quot; } } iex&gt; Handler . run ( fn -&gt; Enum . map ( 1 .. 10_000 , &amp; &amp;1 * 100 ) end , max_heap_bytes : 4096 ) { :error , % Handler.OOM { message : &quot;Process tried to use more than 4096 bytes of memory&quot; } } iex&gt; Handler . run ( fn -&gt; Process . exit ( self ( ) , :i_am_ded ) end ) { :error , % Handler.ProcessExit { message : &quot;Process exited with :i_am_ded&quot; , reason : :i_am_ded } }","ref":"Handler.html#run/2","title":"Handler.run/2","type":"function"},{"doc":"","ref":"Handler.html#t:bytes/0","title":"Handler.bytes/0","type":"type"},{"doc":"","ref":"Handler.html#t:exception/0","title":"Handler.exception/0","type":"type"},{"doc":"","ref":"Handler.html#t:milliseconds/0","title":"Handler.milliseconds/0","type":"type"},{"doc":"","ref":"Handler.html#t:opt/0","title":"Handler.opt/0","type":"type"},{"doc":"","ref":"Handler.html#t:opts/0","title":"Handler.opts/0","type":"type"},{"doc":"Manage a pool of resources used to run dangerous functions The Pool provides both a synchrous interface ( run/3 ) as well as an asynchronous interface ( async/3 and await/1 ) for running functions. Composing Pools This module also provides a way to have limited resources for particular use-cases, that end up sharing bigger pools of resources. Take for instance a hosted multi-tenant application where you can safely use up to 10GB of memory for a particular task, but you don't want any one customer to use the whole pool, so each customer has a limit of just 5GB. You can handle this scenario using the delegate_to configuration. { :ok , _pid } = Handler.Pool . start_link ( % Handler.Pool { max_memory : 10 * 1024 * 1024 * 1024 , name : :shared_pool } ) { :ok , _pid } = Handler.Pool . start_link ( % Handler.Pool { delegate_to : :shared_pool , max_memory : 5 * 1024 * 1024 * 1024 , name : :customer_pool } ) 100 = Pool . run ( :customer_pool , fn -&gt; 10 * 10 end , max_heap_bytes : 100 * 1024 )","ref":"Handler.Pool.html","title":"Handler.Pool","type":"module"},{"doc":"Asynchronously start a job Take a potentially dangerous function and run it in the pool. You'll either get back an ok tuple with a reference you can pass to the await/1 function, or a reject tuple with an exception describing why the function couldn't be started.","ref":"Handler.Pool.html#async/3","title":"Handler.Pool.async/3","type":"function"},{"doc":"Wait for the result of a job kicked off by async","ref":"Handler.Pool.html#await/1","title":"Handler.Pool.await/1","type":"function"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Handler.Pool.html#child_spec/1","title":"Handler.Pool.child_spec/1","type":"function"},{"doc":"Kill all jobs in a pool. Returns the number of killed jobs. The client that kicked off the work will receive an {:error, exception} as the result for the job. If you don't pass an exception, you will get back a Handler.ProcessExit{reason: :user_killed} by default.","ref":"Handler.Pool.html#flush/2","title":"Handler.Pool.flush/2","type":"function"},{"doc":"Kill jobs by their :name When kicking off a job the :name option can be set and then later this function can be used to kill any jobs in progress with a :name matching the task_name of this function. The client that kicked off the work will receive an {:error, exception} as the result for the job. If you don't pass an exception, you will get back a Handler.ProcessExit{reason: :user_killed} by default.","ref":"Handler.Pool.html#kill/3","title":"Handler.Pool.kill/3","type":"function"},{"doc":"Kill a job by its ref When kicking off a job with the async/3 function a ref is returned and that job can later be killed. The client that kicked off the work will receive an {:error, exception} as the result for the job. If you don't pass an exception, you will get back a Handler.ProcessExit{reason: :user_killed} by default.","ref":"Handler.Pool.html#kill_by_ref/3","title":"Handler.Pool.kill_by_ref/3","type":"function"},{"doc":"Run a potentially dangerous function in the pool This function has pretty much the same interface as Handler.run/2 with the addition of the {:reject, t:exception()} return values when the pool does not have enough resources to start a particular function.","ref":"Handler.Pool.html#run/3","title":"Handler.Pool.run/3","type":"function"},{"doc":"","ref":"Handler.Pool.html#start_link/1","title":"Handler.Pool.start_link/1","type":"function"},{"doc":"","ref":"Handler.Pool.html#t:exception/0","title":"Handler.Pool.exception/0","type":"type"},{"doc":"","ref":"Handler.Pool.html#t:name/0","title":"Handler.Pool.name/0","type":"type"},{"doc":"","ref":"Handler.Pool.html#t:opt/0","title":"Handler.Pool.opt/0","type":"type"},{"doc":"","ref":"Handler.Pool.html#t:opts/0","title":"Handler.Pool.opts/0","type":"type"},{"doc":"","ref":"Handler.Pool.html#t:pool/0","title":"Handler.Pool.pool/0","type":"type"},{"doc":"","ref":"Handler.Pool.html#t:t/0","title":"Handler.Pool.t/0","type":"type"},{"doc":"","ref":"Handler.OOM.html","title":"Handler.OOM","type":"exception"},{"doc":"","ref":"Handler.OOM.html#t:t/0","title":"Handler.OOM.t/0","type":"type"},{"doc":"","ref":"Handler.Pool.InsufficientMemory.html","title":"Handler.Pool.InsufficientMemory","type":"exception"},{"doc":"","ref":"Handler.Pool.InsufficientMemory.html#t:t/0","title":"Handler.Pool.InsufficientMemory.t/0","type":"type"},{"doc":"","ref":"Handler.Pool.NoWorkersAvailable.html","title":"Handler.Pool.NoWorkersAvailable","type":"exception"},{"doc":"","ref":"Handler.Pool.NoWorkersAvailable.html#t:t/0","title":"Handler.Pool.NoWorkersAvailable.t/0","type":"type"},{"doc":"","ref":"Handler.Pool.Timeout.html","title":"Handler.Pool.Timeout","type":"exception"},{"doc":"","ref":"Handler.Pool.Timeout.html#t:t/0","title":"Handler.Pool.Timeout.t/0","type":"type"},{"doc":"","ref":"Handler.ProcessExit.html","title":"Handler.ProcessExit","type":"exception"},{"doc":"","ref":"Handler.ProcessExit.html#t:t/0","title":"Handler.ProcessExit.t/0","type":"type"},{"doc":"","ref":"Handler.Timeout.html","title":"Handler.Timeout","type":"exception"},{"doc":"","ref":"Handler.Timeout.html#t:t/0","title":"Handler.Timeout.t/0","type":"type"},{"doc":"Got a function that might get out of hand? Let the Handler run it in its own process and it will &quot;take care of&quot; the process if things get out of hand.","ref":"readme.html","title":"Handler","type":"extras"},{"doc":"Handler . run ( fn -&gt; 1 + 1 end ) # =&gt; 2 Handler . run ( fn -&gt; :timer . sleep ( 61_000 ) end ) # =&gt; {:error, %Handler.Timeout{message: &quot;process ran for longer than 60000ms&quot;}} Handler . run ( fn -&gt; load_tons_of_data_into_memory ( ) end ) # =&gt; {:error, %Handler.OOM{message: &quot;process tried to use more than 10485760 bytes of memory&quot;}} Handler . run ( fn -&gt; Process . exit ( self ( ) , :i_am_ded ) end ) # =&gt; {:error, %Handler.ProcessExit{message: &quot;process exited with :i_am_ded&quot;}} By default Handler puts a limit of 60sec and 10MB of heap space on the function you are running. You can change these limits by passing the optional second argument. # Set 100MB heap limit and 2sec time limit Handler . run ( fn -&gt; 1 + 1 end , max_heap_bytes : 100 * 1024 * 1024 , max_ms : 2_000 ) # =&gt; 2","ref":"readme.html#usage","title":"Handler - Usage","type":"extras"},{"doc":"You can also use Handler to manage a shared pool of resources. If there are not enough resources available to run your function, you'll get back a {:reject, exception} value. First start a supervised pool somewhere in your application tree. elixir application.ex config = %Handler.Pool{ max_workers: 20, max_memory_bytes: 5 * 1024 * 1024 * 1024, name: :my_pool } children = [ {Handler.Pool, config} ] opts = [strategy: :one_for_one, name: Peg.Supervisor] Supervisor.start_link(children, opts) Now that you have a pool being supervised, you can kick off some where from other parts of your project. Handler.Pool . run ( :my_pool , fn -&gt; danger_will_robinson ( ) end , [ max_heap_bytes : 1024 * 1024 ] )","ref":"readme.html#pool-limits","title":"Handler - Pool Limits","type":"extras"}]